#include "game.h"

Mesh::Mesh(char *filename, int texture, float scale)
{
	texId = texture;

	FILE *f = fopen(filename, "rt");
	fscanf(f, "%i", &vertexNb);
	vertexNb *= 3;

	data = new float[vertexNb * 8];

	for(int i=0; i<vertexNb*8; i++)
		fscanf(f, "%f", &data[i]);

	for(int i=0; i<vertexNb; i++)
	{
		data[i*8+5] *= scale;
		data[i*8+6] *= scale;
		data[i*8+7] *= scale;
	}

	fclose(f);
}

Mesh::~Mesh()
{
	delete[] data;
}


void Mesh::Render()
{
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, texId);

//	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
//	glEnableClientState(GL_NORMAL_ARRAY);
//	glEnableClientState(GL_VERTEX_ARRAY);

	glInterleavedArrays(GL_T2F_N3F_V3F, 0, data);
	glDrawArrays(GL_TRIANGLES, 0, vertexNb);

//	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
//	glDisableClientState(GL_NORMAL_ARRAY);
//	glDisableClientState(GL_VERTEX_ARRAY);
}


////////////////////////////////////////////////////////////////////////

void Skybox::Render()
{
	glDisable(GL_LIGHTING);
	//glDisable(GL_FOG);

	glColor4f(1.,1.,1.,1.);


	glPushMatrix();
	float m[16];
	glGetFloatv(GL_MODELVIEW_MATRIX, m);
	m[12] = m[13] = m[14] = 0.f;
	glLoadMatrixf(m);

	static float rot=0.f;
	glRotatef(rot, 0,1,0);
	rot += .01f;


	for(int i=0; i<6; i++)
	{
		glBindTexture(GL_TEXTURE_2D, tex+i);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

		glPushMatrix();
		switch(i)
		{
		case 0:
		case 1:
		case 2:
		case 3:
			glRotatef(90*i, 0,1,0);
			break;
		case 4:
			glRotatef(90, 1,0,0);
			break;
		case 5:
			glRotatef(-90, 1,0,0);
			break;
		}

		glBegin(GL_TRIANGLE_FAN);

		glTexCoord2f(1.f, 1.f);
		glVertex3f(-s, -s, s);

		glTexCoord2f(1.f, 0.f);
		glVertex3f(-s, s, s);

		glTexCoord2f(0.f, 0.f);
		glVertex3f(s, s, s);

		glTexCoord2f(0.f, 1.f);
		glVertex3f(s, -s, s);

		glEnd();

		glPopMatrix();
	}

	glPopMatrix();
	glEnable(GL_LIGHTING);
	glEnable(GL_FOG);


};
