#include "game.h"

FONT *gFont;
int gTextures[MAX_TEXTURES];
char *textureFiles[MAX_TEXTURES+1] =
{
	"data/star.tga",
	"data/floortext.tga",
	"data/floortext3.tga",
	"data/sky_n.bmp",
	"data/sky_w.bmp",

	"data/sky_s.bmp",
	"data/sky_e.bmp",
	"data/sky_d.bmp",
	"data/sky_u.bmp",
	"data/star.tga",

	"data/ship.tga",
	"data/speedup.tga",
	"data/jump.tga",

	NULL
};

struct tile
{
	char *filename;
	int texnum;
};

Mesh *gTiles[MAX_TILES];
tile tileFiles[MAX_TILES+1] =
{
	{ "data/floor.msh", 1 },
	{ "data/floor2.msh", 2 },
	{ "data/floor4.msh", 1 },
	{ "data/jump.msh", 12 },
	{ NULL, -1 }
};

Mesh *gItems[MAX_ITEM];
tile itemFiles[MAX_ITEM+1] =
{
	{ "data/star.msh", 9 },
	{ "data/speedup.msh", 11 },
	{ NULL, -1 }
};

FSOUND_SAMPLE *gSounds[MAX_SOUNDS];
char * soundFiles[MAX_SOUNDS+1] =
{
	"data/snd/plok.wav",
	"data/snd/tom.wav",
	"data/snd/engineloop.wav",
	"data/snd/falling.wav",
	"data/snd/crash.wav",

	"data/snd/time.wav",
	"data/snd/beep.wav",
	"data/snd/ready.wav",
	"data/snd/go.wav",
	"data/snd/gameover.wav",
	NULL
};


char *gTitleText[] =
{
  //"1234567890123456789012345678901234567890"
	"",
	"",
	"            -= Ring 48 =-",
	"              by Codexus",
	"",
	"       Ludum Dare's October 2004",
	"48 Hours game programming competition",
	"",
	"  \\\\\\ PRESS SPACEBAR TO BEGIN ///",
	"       (hold H for instructions)",
	"",
	"",
	"",
	NULL
};

char *gHelpText[] =
{
  //"1234567890123456789012345678901234567890"
	"",
	" -= Ring 48 =-",
	"   by Codexus",
	"",
	"",
	" How to play:",
	"   - Control the ship with the cursor keys.",
	"   - Avoid the holes and the obstacles",
	"   - You will not fall if you have    ",
	"     enough speed to jump across holes",
	"   - Stars are worth points",
	"   - Power-ups increase maximum speed",
	"   - 5 laps = time extended",
	NULL
};

Mesh *toto;
RandomMap *testmap;
ItemMap *itemmap;
Camera *camera;
Skybox *sky;
Player *player;

float ringangle;

char *gMessage = NULL;
float msgTimer = 0.f;

int gGameMode;

FSOUND_STREAM *gMusic;


float globLightAmb[] = {.3f, .3f, .3f, 1.f};
float globLightDif[] = {2.f, 2.f, 1.5f, 1.f};
float globLightPos[] = {-500.f, 500.f, -500.f, 1.f};

float globLightAmb2[] = {.7f, .7f, .7f, 1.f};
float globLightDif2[] = {.5f, .6f, 1.f, 1.f};
float globLightPos2[] = {500.f, -500.f, 500.f, 1.f};

float gSkyColor[] = {.44f, .51f, .64f, .3f};


#define MAX_TIME 60.f
#define MAX_LIFE 9



void LoadTextures()
{
	BITMAP *bmp;

	for(int i=0; textureFiles[i]; i++)
	{

		bmp = load_bitmap(textureFiles[i], NULL);
		gTextures[i] = allegro_gl_make_texture_ex(AGL_TEXTURE_MIPMAP, bmp, -1);
		destroy_bitmap(bmp);
	}
}

void LoadSounds()
{
	for(int i=0; soundFiles[i]; i++)
	{
		gSounds[i] = FSOUND_Sample_Load(FSOUND_FREE,soundFiles[i],0,0,0);
	}
}

void LoadMeshes(tile *meshfiles, Mesh **meshes)
{
	for(int i=0; meshfiles[i].filename; i++)
		meshes[i] = new Mesh(meshfiles[i].filename, gTextures[meshfiles[i].texnum]);
}

void Game::Init()
{
	// get a font ready
	gFont = allegro_gl_load_system_font("Courier New", AGL_FONT_TYPE_OUTLINE, 100, 30);

	glMatrixMode(GL_PROJECTION);
	gluPerspective(60.f, 4.f/3.f, 20.f, 20000.f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glEnable(GL_CULL_FACE);

	glDepthFunc(GL_LEQUAL);
	glEnable(GL_DEPTH_TEST);

	glLightfv(GL_LIGHT1, GL_DIFFUSE, globLightAmb);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, globLightDif);
	glEnable(GL_LIGHT1);

	glLightfv(GL_LIGHT2, GL_DIFFUSE, globLightAmb2);
	glLightfv(GL_LIGHT2, GL_DIFFUSE, globLightDif2);
	glEnable(GL_LIGHT2);

	glEnable(GL_LIGHTING);

	glClearColor(gSkyColor[0], gSkyColor[1], gSkyColor[2], gSkyColor[3]);
	glFogfv(GL_FOG_COLOR, gSkyColor);
	glFogf(GL_FOG_START, 5.f);
	glFogf(GL_FOG_END, 5000.f);
	glFogi(GL_FOG_MODE, GL_LINEAR);
	glEnable(GL_FOG);

	LoadTextures();
	LoadMeshes(tileFiles, gTiles);
	LoadMeshes(itemFiles, gItems);
	LoadSounds();

	FSOUND_Sample_SetMode(gSounds[2], FSOUND_LOOP_NORMAL);

	gGameMode = GMODE_TITLE;

	player = new Player;

	testmap = new RandomMap(60,10,500);
	itemmap = new ItemMap(testmap);

	camera = new Camera(Vector(-64.06,-900,-26.3));
	camera->angle = 49.76;
	camera->zoom = 1283;

	sky = new Skybox(gTextures[3], 3000.f);

   gMusic=FSOUND_Stream_Open("data/snd/zik.mp3",0, 0, 0);
   FSOUND_Stream_SetMode(gMusic, FSOUND_LOOP_NORMAL);
   FSOUND_Stream_Play(1,gMusic);
}

int Game::Loop(float ticks)
{
	if(gGameMode==GMODE_TITLE)
	{
		if(key[KEY_SPACE])
		{
			gGameMode=GMODE_READY;
			player->Ready();
		}
	}
	if(key[KEY_ESC])
	{
		return false;
		//gGameMode=GMODE_QUIT;
	}

	glClear(GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();

	if(gGameMode==GMODE_TITLE)
	{
		camera->rotate += .1f;

//		camera->Update(ticks);
		camera->Transform();
	}

	if(gGameMode==GMODE_PLAYING)
	{
		player->Update(ticks);
		player->ChaseCamera();
		player->PowerUps();
	}

	if(gGameMode==GMODE_GAMEOVER)
	{
		player->GameOverWait(ticks);
		player->ChaseCamera();
	}

	if(gGameMode==GMODE_READY)
	{
		player->ReadyWait(ticks);
		player->ChaseCamera();
	}

	glLightfv(GL_LIGHT1, GL_POSITION, globLightPos);
	glLightfv(GL_LIGHT2, GL_POSITION, globLightPos2);

	player->Render();

	testmap->Render(gTiles, 100.f);
	itemmap->Render(gItems, 100.f, 25.f);

	sky->Render();

	// Ortho mode
	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();

	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_TEXTURE_2D);

	glLoadIdentity();
	gluOrtho2D(0,20, 0,20);

	// fps counter
	static float fps_ticks = 0.f;
	static int frames = 0, fps = 0;
	fps_ticks += ticks;
	frames++;
	if(fps_ticks>=1.f)
	{
		fps_ticks -= 1.f;
		fps = frames;
		frames = 0;
	}

	if(gGameMode==GMODE_TITLE)
	{
		if(key[KEY_H])
		{
			glPushMatrix();
			glScalef(.8,.8,.8);
			for(int i=0; i<20 && gHelpText[i]; i++)
				allegro_gl_printf(gFont, 0,20-i,0, 0xffffff, gHelpText[i]);
			glPopMatrix();
		}
		else
		{
			for(int i=0; i<20 && gTitleText[i]; i++)
				allegro_gl_printf(gFont, 0,19-i,0, 0xffffff, gTitleText[i]);
		}

		glPushMatrix();
		glScalef(.5,.5,.5);
		allegro_gl_printf(gFont, 23,0,0, 0xffffff, "(c)copyright 2004 Marc Schefer");
		glPopMatrix();
	}
	else
	{
		if(msgTimer>0.f && gMessage)
		{
			msgTimer -= ticks;
			glPushMatrix();
			glScalef(2,2,2);
//			allegro_gl_printf(gFont, max(0,10-strlen(gMessage)),5,0, 0xffffff, gMessage);
			allegro_gl_printf(gFont, 2,5,0, 0xffffff, gMessage);
			glPopMatrix();
		}

			glPushMatrix();
//			glScalef(2,2,2);
			allegro_gl_printf(gFont, 0,19,0, 0xffffff,"Score: %05i Life: %i/9 Time: %02i %i/5", player->score, player->life, player->seconds_left, (player->laps%5)+1);
			glPopMatrix();

	}



	//allegro_gl_printf(gFont, 1,1,0, 0xffffff, "%i %f %f %i %i", fps, player->time, player->booster, player->score, player->laps);

	glScalef(.5,.5,.5);
	allegro_gl_printf(gFont, 0,0,0, 0xffffff, "%i fps", fps);

	glEnable(GL_LIGHTING);
	glEnable(GL_DEPTH_TEST);

	// Back to perspective mode
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);

	allegro_gl_flip();

	return gGameMode!=GMODE_QUIT;
}

Camera::Camera(Vector position)
{
	pos = position;
	speed = Vector(0,0,0);
	rotate = rotspeed = 0.f;
	zoom = 500.f;
	zoomspeed= 0.f;
	angle=30.f;
	anglespeed=0.f;
}

void Camera::Update(float ticks)
{
	Vector accel(0,0,0);
	if(key[KEY_UP])
		accel += Vector(0,0,-1);
	if(key[KEY_DOWN])
		accel += Vector(0,0,1);
	if(key[KEY_LEFT])
		accel += Vector(-1,0,0);
	if(key[KEY_RIGHT])
		accel += Vector(1,0,0);

	accel.RotateY(rotate);
	accel *= 200.f;
	speed += (accel-speed) * ticks * 5;

	float rotaccel=0.f;
	if(key[KEY_DEL])
		rotaccel = 1.f;
	if(key[KEY_PGDN])
		rotaccel = -1.f;
	rotaccel *= 40.f;
	rotspeed += (rotaccel-rotspeed) * ticks * 3;;

	float zoomaccel=0.f;
	if(key[KEY_HOME])
		zoomaccel = -1.f;
	if(key[KEY_END])
		zoomaccel = 1.f;
	zoomaccel *= 150.f;
	zoomspeed += (zoomaccel-zoomspeed) * ticks * 3;

	float angleaccel=0.f;
	if(key[KEY_INSERT])
		angleaccel = 1.f;
	if(key[KEY_PGUP])
		angleaccel = -1.f;
	angleaccel *= 40.f;
	anglespeed += (angleaccel-anglespeed) * ticks * 3;;

	pos += speed * ticks;
	rotate += rotspeed * ticks;
	zoom += zoomspeed * ticks;
	angle += anglespeed * ticks;
}

void Camera::Transform()
{
	glLoadIdentity();
	glTranslatef(0, 0, -zoom);
	glRotatef( angle, 1,0,0);
	glRotatef( rotate, 0,1,0);
	glTranslatef(-pos.x, -pos.y, -pos.z);
}


Player::Player()
{
	playermesh = new Mesh("data/ship.msh", gTextures[10], 1.f);

	pos = Vector(0,50,0);
	speed = Vector(0,0,0);
	rot = Vector(0,-90,0);

	zoom = 200.f;
	camangle=-30.f;
	camheight= 50.f;

	booster = 0.f;
	score = 0;
	laps = 10;
	time = MAX_TIME;
	life = MAX_LIFE;

	ringangle = 0.f;
	currow = 0;
	invincible = 0.f;
	stuck = 0.f;
}

void Player::Update(float ticks)
{
	int x,y;
	testmap->GetMapCoords(player->pos, x, y);

	time -= ticks;
	int new_seconds = int(time);
	if(new_seconds!=seconds_left && new_seconds<10)
		FSOUND_PlaySound(0, gSounds[6]);

	seconds_left = new_seconds;

	if(time<0.f || life<=0)
		GameOver();


	invincible = max(0.f, invincible-ticks);
	stuck = max(0.f, stuck-ticks);
	if(stuck>0.f)
		return;

	int curlaps = int(pos.x / (2.f*PI*RING_RADIUS));
	if(curlaps>laps)
	{
		score += 500;
		if(curlaps%5 == 0)
		{
			// time extended!
			time += min(40.f - float(curlaps) / 5.f * 1.f, 15.f);
			score += 2000;
			FSOUND_PlaySound(2, gSounds[5]);
			gMessage = "Time Extended";
			msgTimer = 2.0f;

		}
		laps = curlaps;
	}

	booster = max(0.f, booster - 80.f*ticks);

	Vector accel( min(speed.x, 500.f),0,0);
	if(key[KEY_UP])
		accel.x = 1000.f+booster;
	else
		booster=max(0.f, booster - 400.f*ticks);

	if(key[KEY_DOWN])
		accel.x = 0.f;
	if(key[KEY_LEFT])// && pos.z<= 360.f)
		accel += Vector(0,0,1);
	if(key[KEY_RIGHT])// && pos.z>= -460.f)
		accel += Vector(0,0,-1);

	float targetheight = testmap->Array(x,y) * 25.f + 25.f;
	if(pos.y <= 10.f || targetheight<26.f)
	{
		if(invincible<=0.f)
			speed.y -= ticks * 1500.f;
	}
	else
	{
		float diff = min(abs(targetheight - pos.y), 20.f);
		accel.y = diff * diff * 10.f;

		float drag = -speed.y*.1f;
		speed.y += drag;

		if(targetheight-pos.y > 0.f )
			speed.y += accel.y * ticks;
		else
			speed.y -= accel.y * ticks;
	}

	if(pos.y <= -100.f)
	{
		pos.y = 50;
		pos.z = 0;
		speed = Vector(0,0,0);
		invincible += 3.f;
		stuck += .5f;
		booster = 0.f;

		FSOUND_PlaySound(0, gSounds[3]);

		gMessage = "ooops";
		msgTimer = 2.0f;

		--life;
	}

	if(testmap->Array(x,y)==2 && invincible<=0.f && pos.y <60.f)
	{
		pos.y = 50;
		pos.z = 0;
		speed = Vector(0,0,0);
		invincible += 3.f;
		stuck += .5f;
		booster = 0.f;

		FSOUND_PlaySound(0, gSounds[4]);

		gMessage = "CRASH!";
		msgTimer = 2.0f;

		--life;

	}

//	if(targetheight-pos.y)
//


	//accel.x += booster;

//	accel.RotateY(rotate);
//	accel.x *= 2000.f;
	accel.z *= 500.f;
	speed.x += (accel.x-speed.x) * ticks * 1;
	speed.z += (accel.z-speed.z) * ticks * 5;

	pos += speed * ticks;

	pos.z = max(pos.z, -520.f);
	pos.z = min(pos.z, 420.f);

	rot.z = -(speed.z / 400.f) * 30.f;
	rot.y = (speed.z / 400.f) * 10.f;


//	if(pos.x > 2.f*PI*RING_RADIUS)
//		pos.x = fmodf(pos.x, (2.f*PI*RING_RADIUS));

	ringangle = 180 + 360.f * .5f * pos.x / (testmap->w*TILE_SIZE);
	playerangle = 360.f * .5f * pos.x / (testmap->w*TILE_SIZE);


	if(x!=currow)
	{
		testmap->RandomRow((currow+4)%testmap->w);
		itemmap->RandomRow((currow+4)%testmap->w);
		currow=x;
	}
}

void Player::Render()
{
	glPushMatrix();

	glRotatef(-playerangle, 0,0,1);

	glTranslatef(0, -RING_RADIUS + pos.y, pos.z);

	glRotatef(rot.y - 90, 0,1,0);
	glRotatef(rot.x, 1,0,0);
	glRotatef(rot.z, 0,0,1);

//	static float rot=0.f;
//	glRotatef(rot, 0,1,0);
//	rot += .1f;

	playermesh->Render();
	glPopMatrix();
}

void Player::ChaseCamera()
{
	glLoadIdentity();
	//glTranslatef(0, 0, -zoom);
	glRotatef( camangle, 1,0,0);
	glRotatef( -90 -rot.y, 0,1,0);

	glTranslatef(0, RING_RADIUS - camheight, -pos.z);

	glRotatef(playerangle-20, 0,0,1);
}

void Player::PowerUps()
{
	int item, x,y;
	testmap->GetMapCoords(player->pos, x, y);
	
	if((item = itemmap->Array(x,y)) !=0)
		itemmap->Array(x,y) = 0;

	switch(item)
	{
	case 1: //star
		FSOUND_PlaySound(0, gSounds[0]);
		score += 100;
		break;
	case 2: // small booster
		FSOUND_PlaySound(0, gSounds[1]);
		booster = min(booster+100, 1000);
		break;
	}
}

void Player::GameOver()
{
	speed = Vector(0,0,0);
	gameover_timer = 3.f;
	gGameMode = GMODE_GAMEOVER;
	FSOUND_StopSound(1);

	gMessage = "GAME OVER";
	msgTimer = 2.9f;

	FSOUND_PlaySound(2, gSounds[9]);
}

void Player::GameOverWait(float ticks)
{
	gameover_timer -= ticks;

	if(gameover_timer<0.f)
	{
		FSOUND_Stream_Play(1, gMusic);
		gGameMode = GMODE_TITLE;
	}
}

void Player::Ready()
{
	ready_timer = 1.5f;
	gGameMode = GMODE_READY;

	gMessage = "READY ?!!";
	msgTimer = 1.4f;

	FSOUND_Stream_Stop(gMusic);
	FSOUND_PlaySound(2, gSounds[7]);

	pos = Vector(0,50,0);
	speed = Vector(0,0,0);
	rot = Vector(0,-90,0);

	zoom = 200.f;
	camangle=-30.f;
	camheight= 50.f;

	booster = 0.f;
	score = 0;
	laps = 0;
	time = MAX_TIME;
	life = MAX_LIFE;

	ringangle = 0.f;
	currow = 0;
	invincible = 0.f;
	stuck = 0.f;

	testmap->Randomize();
	itemmap->Randomize();
	Update(0.001);
}

void Player::ReadyWait(float ticks)
{
	ready_timer -= ticks;

	if(ready_timer<0.f)
	{
		gMessage = "GO !!!!";
		msgTimer = .5f;

		FSOUND_PlaySound(1, gSounds[2]);
		FSOUND_PlaySound(2, gSounds[8]);

		gGameMode = GMODE_PLAYING;
	}
}
