#include "game.h"

Map::Map(char *filename)
{
	FILE *f = fopen(filename, "rt");
	fscanf(f, "%i %i", &w, &h);

	data = new int[w*h];
	for(int i=0; i<w*h; i++)
		fscanf(f, "%i", &data[i]);

	fclose(f);
}

Map::Map(int width, int height)
{
	w = width;
	h = height;
	data = new int[w*h];
	memset(data, 0, sizeof(int)*w*h);
}

Map::~Map()
{
	delete[] data;
}

void Map::Render(Mesh **meshArray, float tileSize)
{
	float offsetX = -w*tileSize / 2.f;
	float offsetY = -h*tileSize / 2.f;


	for(int i=w; (--i)>=0; )
		for(int j=0; j<h; j++)
		{
			glPushMatrix();


			float angle = (i-(w/2))*6 + ringangle;
			glRotatef(angle, 0,0,1);

//			float angle2 = (j-(h/2))*2;
//			glRotatef(angle2, 1,0,0);

			glTranslatef(0, -RING_RADIUS, offsetY+j*tileSize);
//			glTranslatef(offsetX+i*tileSize-tileSize*sinf(angle/360.f*2.f*PI), -1000, offsetY+j*tileSize);
			if(data[j*w+i]>0)
				meshArray[data[j*w+i]]->Render();

			glPopMatrix();
		}

}

void Map::GetMapCoords(Vector pos, int& x, int& y)
{
	x = int(-pos.x / TILE_SIZE) % w;
	x = (x+w)%w;
	y = int((pos.z + 1550) / TILE_SIZE) % h;
}

////////////////////////////////////////////////////////////////////////

//float prob[MAX_LEVEL*2] = {10., 1., 2., 0, 3., 0, 4., 0};
//float prob[MAX_LEVEL*2] = {1, 1, .04, .1, 1., 1, 1., 1};

RandomMap::RandomMap(int width, int height, int iterations)
	: Map(width, height)
{
	size = width * height;
	Randomize();
}



RandomMap::~RandomMap()
{
}

void RandomMap::Randomize()
{
	for(int i=0; i<h; i++)
		Array(0, i) = 1;
	for(int i=w; (--i)>0; )
		RandomRow(i);
}

void RandomMap::RandomRow(int row)
{
	//if(row==0)
	//{
	//	for(int i=0; i<h; i++)
	//		Array(row, i) = 3;
	//	return;
	//}

	for(int i=0; i<h; i++)
		Array(row, i) = 0;

	int count=0;
	for(int i=0; i<h; i++)
	{
		if(Array((row+1)%w, i)==1 && (float(rand()) / RAND_MAX) < .6f)
		{
			Array(row, i) = 1;
			count++;
			continue;
		}
		if((float(rand()) / RAND_MAX) < .1)
		{
			Array(row, i) = 1;
			count++;
			continue;
		}

		if(Array((w+row+1)%w, i)==1 && (float(rand()) / RAND_MAX) < .05f && player->laps>4)
		{
			Array(row, i) = 3;
			continue;
		}

		if((Array((w+row+1)%w, i)==2 || Array((w+row+1)%w, i)==3)&& (float(rand()) / RAND_MAX) < .1f  && player->laps>5)
		{
			Array(row, i) = 2;
			continue;
		}

		if((float(rand()) / RAND_MAX) < .03f  && player->laps>5)
		{
			Array(row, i) = 2;
			continue;
		}
	}
		int mincount;
		if(player->laps<5)
			mincount = 10;
		if(player->laps<10)
			mincount = 8;
		else if(player->laps<20)
			mincount = 7;
		else if(player->laps<30)
			mincount = 5;
		else
			mincount = 4;

		if(count<=mincount)
		{
			for(int j=0; j<mincount; j++)
				Array(row, rand()*h/RAND_MAX) = 1;
		}

}

/*
void RandomMap::Cycle()
{
	for(int i=0; i<(MAX_LEVEL-1); i++)
	{
		if( count[i]>0 && count[i+1]>0 && ((float(rand())/float(RAND_MAX)) < (float(count[i])/float(size) * prob[i*2+1])) )
		{
			int x = FindCanGrow(i, i+1);
			if(x>=0)
			{
				count[i]--;
				count[i+1]++;
				data[x] = i+1;
			}
		}

		if(count[i]>0 && count[i+1]<1 && ((float(rand())/float(RAND_MAX)) < (float(count[i])/float(size) * prob[i*2])) )
		{
			int x;

			if(i>0)
				x = FindCanGrow(i, i);
			else
				x = FindLevel(i);

			count[i]--;
			count[i+1]++;
			data[x] = i+1;
			//break;
		}
	}
}

int RandomMap::FindLevel(int level)
{
	int x = int(float(rand()) * size / RAND_MAX);
	while(data[x]!=level)
	{
		x += 4391; // prime number
		x %= size;
	}
	return x;
}

int RandomMap::FindCanGrow(int level, int growlevel)
{
	int c=0;
	int x = int(float(rand()) * size / RAND_MAX);
	while(c++<size && !(data[x]==level && CanGrow(x, growlevel)>0))
	{
		x += 4391; // prime number
		x %= size;
	}

	if(c>size)
		return -1;

	return x;
}

int RandomMap::CanGrow(int index, int growlevel)
{
	int c=0;

	for(int i=0; i<4; i++)
		if(data[Neighbour(index, i)] == growlevel)
			c++;

	return c;
}

int RandomMap::Neighbour(int index, int neighbour)
{
	switch(neighbour)
	{
	case 0:
		return (index - w) % size;
	case 1:
		return (index+1) % size;
	case 2:
		return (index + w) % size;
	case 3:
		return (index-1) % size;
	default:
		return index;
	}
}
*/
/////////////////////////////////////////////////////77
///
//

float item_prob[MAX_LEVEL][MAX_ITEM] =
{
	{0.f, 0.f},
	{.05f, 0.03f},
	{.02f, 0.f},
	{.01f, 0.f},
};

float item_repeat[MAX_ITEM] = { 0.8f, 0.7f };

ItemMap::ItemMap(RandomMap *parentMap)
	: Map(parentMap->w, parentMap->h)
{
	parent = parentMap;
	Randomize();
}

ItemMap::~ItemMap()
{
}

void ItemMap::Randomize()
{
	for(int i=w; (--i)>0; )
		RandomRow(i);
}

void ItemMap::RandomRow(int row)
{
	for(int i=0; i<h; i++)
		Array(row, i) = 0;

	for(int i=0; i<h; i++)
	{
		for(int j=0; j<MAX_ITEM; j++)
		{
			if(Array((row+1)%w, i)==j+1 && item_prob[parent->Array(row, i)][j]>0 && (float(rand()) / RAND_MAX) < item_repeat[j])
			{
				Array(row, i) = j+1;
				continue;
			}
			if((float(rand()) / RAND_MAX) < item_prob[parent->Array(row, i)][j])
			{
				Array(row, i) = j+1;
				continue;
			}
		}
	}
}

void ItemMap::Render(Mesh **meshArray, float tileSize, float tileHeight)
{
	float offsetX = -w*tileSize / 2.f;
	float offsetY = -h*tileSize / 2.f;

	static float rot=0.f;
	rot += .5f;

	for(int i=0; i<w; i++)
		for(int j=0; j<h; j++)
		{
			glPushMatrix();

//			glTranslatef(offsetX+i*tileSize, parent->data[j*w+i] * tileHeight, offsetY+j*tileSize);

			float angle = (i-(w/2))*6  + ringangle;
			glRotatef(angle, 0,0,1);
			glTranslatef(0, -RING_RADIUS + parent->data[j*w+i] * tileHeight, offsetY+j*tileSize);

			glRotatef(rot+(i+j)*30, 0,1,0);
			if(data[j*w+i]>0)
				meshArray[data[j*w+i]-1]->Render();

			glPopMatrix();
		}
}
