#ifndef MATH_H
#define MATH_H

#include <math.h>

#define PI 3.1415926f

class Vector
{
public:
	float x, y, z;

	Vector() {}
	Vector(float x, float y, float z)
	{
		this->x=x;
		this->y=y;
		this->z=z;
	};

	Vector operator+(const Vector& v) const
	{
		return Vector(x+v.x, y+v.y, z+v.z);
	}

	Vector& operator+=(const Vector& v)
	{
		x += v.x;
		y += v.y;
		z += v.z;
		return *this;
	}

	Vector operator-(const Vector& v) const
	{
		return Vector(x-v.x, y-v.y, z-v.z);
	}

	Vector& operator-=(const Vector& v)
	{
		x -= v.x;
		y -= v.y;
		z -= v.z;
		return *this;
	}

	Vector operator*(float f) const
	{
		return Vector(x*f, y*f, z*f);
	}

	Vector& operator*=(float f)
	{
		x *= f;
		y *= f;
		z *= f;
		return *this;
	}

	void RotateX(float degree)
	{
		float s = sinf(degree / 360.f * 2.f * PI);
		float c = cosf(degree / 360.f * 2.f * PI);

		float nz = z*c - y*s;
		y = z*s + y*c;
		z=nz;
	}

	void RotateY(float degree)
	{
		float s = sinf(degree / 360.f * 2.f * PI);
		float c = cosf(degree / 360.f * 2.f * PI);

		float nx = x*c - z*s;
		z = x*s + z*c;
		x=nx;
	}

	void RotateZ(float degree)
	{
		float s = sinf(degree / 360.f * 2.f * PI);
		float c = cosf(degree / 360.f * 2.f * PI);

		float nx = x*c - y*s;
		y = x*s + y*c;
		x=nx;
	}


};

#endif
